package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.DataSourceName;
import pwc.taxtech.atms.constant.enums.EnumOperationType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.constant.enums.KeyValueConfigResultType;
import pwc.taxtech.atms.dpo.CitAssetDetailResultDto;
import pwc.taxtech.atms.dto.vatdto.AssetDetailDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.vat.dpo.AssetDetailResultDto;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/// <summary>
/// 名称：资产
/// 功能：从已经分类的资产数据中获取对应的数据
/// </summary>
/// <param name="assetType">资产类别：1，固定资产；2，长期待摊；3，无形资产</param>
/// <param name="assetDetailType">资产二级分类：见数据库AssetDetailGroup表DetailGroupType字段</param>
/// <param name="dataType">
/// 取值列：1，原值（财务）；2，本年折旧额（财务）；3，累计折旧额（财务）；4，每月折旧额（财务）；5，本年年终剩余价值（财务）
/// 6，每月折旧额（税务）；7，截止本年累计折旧额（税务）；8，本年折旧额（税务）；9，累计差异（税务），10，本年差异（税务）
/// </param>
/// <param name="year">年份：0，当年；-1，上一年；1，下一年（以此类推到10）</param>
/// <returns></returns>
public class ZC extends FunctionBase implements FreeRefFunction {

    public ZC(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 4) {
            return NumberEval.ZERO;
        }

        int assetType = getIntParam(args[0], ec);
        int assetDetailType = getIntParam(args[1], ec);
        int dataType = getIntParam(args[2], ec);
        int year = getIntParam(args[3], ec);

        String formulaExpression = "ZC(" + assetType + "," + assetDetailType + ","
                + dataType + "," + year + ")";
        logger.debug(formulaExpression);

        int curYear = formulaContext.getYear() + year;

        BigDecimal totalData = new BigDecimal("0.00");

        List<ReportCellDataSourceDto> dataSource = new ArrayList<>();
        ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
        try {
            CitAssetDetailResultDto assetDetailResultDto =
                    SpringContextUtil.assetsListMapper.getCitAssetDetialResult(assetType, assetDetailType,formulaContext.getProjectId());

            dto.setRowIndex(ec.getRowIndex());
            dto.setColumnIndex(ec.getColumnIndex());
            dto.setPeriod(formulaContext.getPeriod());
            dto.setReportTemplateId(formulaContext.getReportTemplateGroupId().toString());
            dto.setYear(formulaContext.getYear());
            dto.setIsOnlyManualInput(Boolean.FALSE);

//            dto.setType(FormulaDataSourceType.AssetListSource.getCode());
            if(assetDetailResultDto != null){
                switch (dataType) {
                    case 1:
                        totalData = assetDetailResultDto.getAssetsList().getAccountAcquisitionValue();
                        break;
                    case 2:
                        totalData = assetDetailResultDto.getAssetsList().getAccountYearDepreciationAmount();
                        break;
                    case 3:
                        totalData = assetDetailResultDto.getAssetsList().getAccountTotalDepreciationAmount();
                        break;
                    case 4:
                        totalData = assetDetailResultDto.getAssetsList().getAccountMonthDepreciationAmount();
                        break;
                    case 5:
                        totalData = assetDetailResultDto.getAssetsList().getYearEndValue();
                        break;
                    case 6:
                        totalData = assetDetailResultDto.getAssetsList().getTaxMonthDepreciationAmount();
                        break;
                    case 7:
                        totalData = assetDetailResultDto.getAssetsList().getTaxToCurrentYearDepreciationAmount();
                        break;
                    case 8:
                        totalData = assetDetailResultDto.getAssetsList().getTaxCurrentYearDepreciationAmount();
                        break;
                    case 9:
                        totalData = assetDetailResultDto.getAssetsList().getTotalDifferenceAmount();
                        break;
                    case 10:
                        totalData = assetDetailResultDto.getAssetsList().getYearDifferenceAmount();
                        break;
                    default:
                        break;
                }
            }
            dto.setAmount(totalData);
            dataSource.add(dto);
            return new NumberEval(totalData.doubleValue());
        }catch (Exception e){
            e.printStackTrace();
            logger.error("计算ZC公式报错");
            dto.setAmount(new BigDecimal("0.00"));
            return new NumberEval(0.00);
        }finally {
            Long dataSourceId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.AssetDetailDataSourceDto,
                    totalData, formulaContext.getPeriod(), formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(formulaContext.getPeriod(), ec, formulaExpression, totalData, dataSourceId, formulaContext.getProjectId());
        }


    }
}
